#!/usr/bin/bash
#
# ./build_tasks.sh [git repo | path]
#
# Unless tasks are specified, we'll look for them in:
#
#   1. /var/lib/bughunting/tasks
#
#   2. ./tasks # copied
#
#   3. https://gitlab.com/bughunting/tasks.git
#
#
# Optional
#   git repo	url to the git repo containing tasks
#   path	    custom path to the tasks
#

# Handy for debugging purposes
pwd

set -xe
bash -n "$0"

abort () {
  echo "Error:" "$@" >&2
  exit 1
}

my_dir="`dirname "$(readlink -e "$0")"`"
[[ -d "$my_dir" ]] || abort "Unknown directory: $my_dir"

sources="${my_dir}/setup_sources.sh"
[[ -x "$sources" ]] || abort "Setup sources script not found or not executable: $sources"

# VARS
clone=
tasks=
tasks_dir='/var/lib/bughunting/tasks'
tasks_repo='https://gitlab.com/bughunting/tasks.git'

# ARGS
tasks="$1"
shift ||:

if [ -z "$tasks" ] ; then
  if [ -r "${tasks_dir}/Makefile" ]; then
  	tasks="$tasks_dir"

  elif [ -r "./Makefile" ]; then
    tasks="."

  elif [ -d "tasks" ]; then
  	tasks='tasks'

  else
  	clone="$tasks_repo"

  fi

elif [ -n "$(grep -E '^(https|ssh|git)\:\/\/' <<< "$tasks" ; grep -E '^git\@' <<< "$tasks")" ] ; then
	clone="$tasks"

fi

if [ -n "$clone" ] ; then
	# clone into folder named tasks
	git clone "$clone" tasks
  tasks=tasks
fi

pushd "$tasks"
  rm -rf rpm-build-dir ||:

  make srpm
  dnf --nogpgcheck --nodocs builddep -y rpm-build-dir/*.src.rpm
  make rpms

  dnf --nogpgcheck --nodocs install -y rpm-build-dir/*/bughunting-tasks-*.rpm ||:
  dnf --nogpgcheck --nodocs downgrade -y rpm-build-dir/*/bughunting-tasks-*.rpm ||:
  dnf --nogpgcheck --nodocs update -y rpm-build-dir/*/bughunting-tasks-*.rpm ||:
  dnf --nogpgcheck --nodocs reinstall -y rpm-build-dir/*/bughunting-tasks-*.rpm

  rpm -V bughunting-tasks

  rm -rf rpm-build-dir ||:
popd

# Setup tasks in /var/lib/bughunting/tasks = default
$sources

exit 0
