#!/usr/bin/zsh

set -e
bash -n "$0"

die () {
  echo "Error:" "$@" >&2
  exit 1
}

git status -uno | grep -q '^nothing to commit ' \
  || die 'You have uncommited changes!'

p=""
while [[ "${1:0:1}" == '-' && "${1:0:2}" != '--' ]]; do
  [[ "$1" == "$p" ]] && die "Unknown arg: $1"
  p="$1"

  [[ "$1" == "-c" ]] && {
    C=" --build-arg CODENAME=$2 --build-arg KEY=$3"
    shift 3
    :
  }

  [[ "$1" == "-p" ]] && {
    F="$2"
    shift 2
    :
  }

  [[ "$1" == '-r' ]] && {
    [[ -r Makefile ]] || die "Current folder is invalid."

    set +e
    rm -f *.txz; rm -f *.src.rpm ; rm -rf rpm-build-dir; rm -rf ~/rpmbuild/
    set -e

    make archive; make srpm; make rpms

    shift
  }

  [[ "$1" == "-t" ]] && {
    T="$2"
    shift 2
    :
  }
  :
done

[[ -z "$1" ]] && {
  [[ -n "$T$C$F" ]] && die "Args cannot be combined with default build."
  exec "$0" server client
}
[[ "$1" == '--all' ]] && {
  [[ -n "$T$C$F" ]] && die "Args cannot be combined with '--all'."
  exec "$0" server devel client sshd #summer
}

# Default prefix
F="${F:-bughunting}"

[[ -n "$F" ]] || die "Missing prefix."
[[ -n "$1" ]] || die "Missing image."

echo "${@}" \
  | tr -s ' ' '\n' \
  | grep -v "^$" \
  | xargs -ri zsh -c "
      set -x

      # Squash even the base layers
      grep -qE '^(server|client|devel)$' <<< \"{}\" \
        && sq='-all' \
        || sq=

      [[ -n '$T' ]] \
        && im='$T' \
        || im=\"{}\"

      im=\"${F}/\${im}\"

      grep ':' <<< \"\${im}\" \
        || im=\"\${im}:latest\"

      podman rmi -f \"\${im}\"

      podman build$C --squash\$sq -f 'Dockerfile.{}' -t \"\$im\" \
        || exit 255
  "
