#!/usr/bin/bash

bash -n "$0" || exit 2

die () {
  echo -e "Error:" "$@" >&2
  exit 1
}

[[ -n "$DEBUG" ]] && set -x

[[ "$NOHUNT" == '1' ]] || NOHUNT='0'
[[ "$TESTING" == '1' ]] || TESTING='0'
[[ "$NOGIT" == '1' ]] || NOGIT='0'
[[ "$OWN" == '1' ]] || OWN='0'

[[ -n "$SOURCES" ]] || die "Sources dir not set, please ask for help."

[[ -n "$KEY" ]] && {
  # 17th character is \n
  [[ $(wc -c <<< "$KEY") -eq 17 ]] \
    || die "KEY should have 16 characters, please ask for help you don't know the KEY."
  :
} || {
  [[ "$NOHUNT" == '1' ]] \
    || die "You have to specify KEY environment variable, like so:\n\n  $ docker run -e KEY=????? ... \n"
}

CHECKKEY="`cat /etc/bughunting/key 2>/dev/null`" ||:

[[ -n "$CHECKKEY" ]] && {
  [[ "$CHECKKEY" == "$KEY" ]] || die "Invalid KEY: $KEY"
}

# Run only if SOURCES dir is empty
ls -d ${SOURCES}/* &>/dev/null || {
  mkdir -p "$SOURCES"

  [[ "$NOHUNT" == '1' ]] && {
    sources_dir="/var/lib/bughunting/sources"
    cd "$sources_dir"

    # Copy the installed tasks into the client's home
    for sources_path in $(ls -d */); do
      cd "$sources_dir"

      source_name="$(basename "$sources_path")"
      cp -r "${sources_path}" "${SOURCES}/${source_name}"

      [[ "$NOGIT" == '1' ]] || {
        : "running git pull: '${SOURCES}/$source_name'"

        cd "${SOURCES}/$source_name" \
          && O="$(git pull 2>&1)" \
          || echo "Warning: Failed to pull source '$source_name', please report it." >&2

        echo "$O" | grep -v '^Already up to date.$'
      }
    done
    :
  } || {

    tasks_dir="${TASKS:-/var/lib/bughunting/tasks}"
    ls -d ${tasks_dir}/*/ &>/dev/null || die "Invalid tasks dir: $tasks_dir, please ask for help."

    cd "$tasks_dir"

    # Copy the installed tasks into the client's home
    for task_path in $(ls -d */); do
      task_name="$(basename "$task_path")"
      cp -r "${task_path}source" "${SOURCES}/$task_name" || die "Failed to copy task: $task_path"
    done

    [[ "$NOGIT" == '1' ]] || {
      # In copy .git/ folder
      git_path="`ls -d .git/`"

      [[ -d "$git_path" ]] && {
        git_name="$(basename "$git_path")"
        cp -r "${git_path}" "${SOURCES}/$git_name" || die "Failed to copy git dir: $git_path"
      }
    }

    # guess from sources path: /home/USER/sources
    user="`cut -d'/' -f3 <<< "$SOURCES"`"
    grep -q "^$user:" /etc/passwd \
      && [[ "$OWN" == '1' ]] && chown -R "$user:" "$SOURCES"

    [[ "$NOGIT" == '1' ]] && {
      : "remove leftover .git directory: '${SOURCES}/.git'"
      rm -rf '${SOURCES}/.git' &>/dev/null || :

    } || {
      : "running git pull: '${SOURCES}/$source_name'"

      cd "$SOURCES" \
        && {
          e=0

          {
            git fetch || e=1
            [[ "$TESTING" == '1' ]] || {
              git reset --hard origin/main || e=1
            }
            git pull || e=1

          } &>/dev/null

          :
        } || e=1

        [[ $e -eq 0 ]] \
          || echo "Warning: Failed to 'git fetch' sources, please report it." >&2

    }
  }
}

# Checks are not needed as well
[[ "$NOHUNT" == '1' ]] || {
  echo -e "\n--> Don't forget to register first!!!"

  ETC=/etc/bughunting
  SERVER="$(cat $ETC/client.yml | grep -A 1 '^server:' | grep '^\s*host' | tr -s ' ' | cut -d' ' -f3)"
  [[ -z "$SERVER" ]] && echo 'Invalid client config!'

  [[ -r "$ETC/web.yml" || -r "$ETC/server.yml" ]] && {
    echo 'Note: development version'

    PORT="$(cat /etc/bughunting/web.yml | grep '^\s*port:' | tr -s ' ' | cut -d' ' -f3)"
    [[ -z "$PORT" ]] && echo 'Invalid server port!'

    echo -e "\nhttp://$SERVER:$PORT/registration/"
  }
}

cd "${SOURCES}"
[[ $# -gt 0 ]] && exec "$@"

echo "Sleeping indefinitely, use '$ docker exec ...' instead."
sleep infinity
