#!/usr/bin/bash
#
# ./setup_sources.sh [path]
#
# Unless tasks are specified, we'll look for them in:
#
#   1. /var/lib/bughunting/tasks
#
#   2. ./tasks # copied
#
# Optional
#   git repo	url to the git repo containing tasks
#   path	    custom path to the tasks
#

# Handy for debugging purposes
pwd

set -xe
bash -n "$0"

abort () {
  echo "Error:" "$@" >&2
  exit 1
}

# VARS
clone=
tasks=
tasks_dir='/var/lib/bughunting/tasks'

# ARGS
tasks="$1"
shift ||:

if [ -z "$tasks" ] ; then
  if [ -d "${tasks_dir}" ]; then
  	tasks="$tasks_dir"

  elif [ -d "tasks" ]; then
  	tasks='tasks'

  else
  	abort "No task dir found."

  fi

elif [ -n "$(grep -E '^(https|ssh|git)\:\/\/' <<< "$tasks" ; grep -E '^git\@' <<< "$tasks")" ] ; then
	clone="$tasks"

fi

if [ -n "$clone" ] ; then
	# clone into folder named tasks
	git clone "$clone" tasks
  tasks=tasks
fi

cd "$tasks"
pwd

ls -d */source

ls -d */task.yml | cut -d'/' -f1 \
  | xargs -ri bash -c "cd '{}/source' || exit 255; grep '^setup_commands:' '../task.yml' | cut -d'[' -f2 | rev | cut -d']' -f2 | rev | tr -s ',' '\n' | sed -e 's/^\s*//g' | sed -e 's/\s*$//g' | grep -v ^$ | sort -u | cut -d\"'\" -f2 | while read C ; do C=\"\$C\"; echo \"> \$(pwd) :: \${C}\"; set -x; eval \"\${C}\"; { set +x ; } &>/dev/null; done"

exit 0
