#--
# Copyright (c) 2011-2016 Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#++

class Players
  def initialize(database, config)
    @db = database
    @config = config
    setup_schema
  end

  def name(player)
    @db.get_first_value "SELECT name FROM players WHERE player like ?", [player]
  end

  def register(machine, name, email, key, terms_of_service, multiple = false)
    player = @config["players"].detect { |pl, ma| ma.to_s == machine && pl }

    $stderr.puts "Register: #{machine}, #{name}, #{email}, #{key} -> #{player}"

    if player
      player = player.first.to_s

      if player == key.downcase
        if name(player)
          player = nil unless multiple
        else
          #begin
            @db.execute "INSERT INTO players (player, name, email, terms_of_service) VALUES (?, ?, ?, ?)",
              [player, name, email, terms_of_service]
          #rescue SQLite3::ConstraintException
            # Constraint index
            #player = nil
          #end
        end
      else
        player = nil
      end
    end
    player
  end

  private

  def setup_schema
    @db.execute "CREATE TABLE IF NOT EXISTS players (
      id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
      created_at DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
      player VARCHAR NOT NULL,
      name VARCHAR NOT NULL,
      email VARCHAR NOT NULL,
      terms_of_service BOOLEAN NOT NULL,
      UNIQUE (player)
    )"
  end
end
